/*
 *                 - Mellanox Confidential and Proprietary -
 * 
 * Copyright (C) Jan 2013, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 * 
 * Except as specifically permitted herein, no portion of the information,
 * including but not limited to object code and source code, may be reproduced,
 * modified, distributed, republished or otherwise exploited in any form or by
 * any means for any purpose without the prior written permission of Mellanox
 * Technologies Ltd. Use of software subject to the terms and conditions
 * detailed in the file "LICENSE.txt".
 */

#ifndef __MTCR_BSD_H__
#define	__MTCR_BSD_H__

#include <errno.h>
#include <sys/types.h>
#include <sys/fcntl.h>

#include <sys/param.h>
#include <ctype.h>
#include <err.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/pciio.h>
#include <sys/queue.h>

//#include <dev/pci/pcireg.h>

#ifdef CONFIG_MTCR_DEBUG
    #define MTCR_DBG(format, arg...) printf(format, ## arg);
#else
    #define MTCR_DBG(format, arg...)
#endif

#ifdef __cplusplus
extern "C" {
#endif


/*
 * Types
 */
#define u64     uint64_t
#define u32     uint32_t
#define u16     uint16_t
#define u8      uint8_t

#define __be64  uint64_t
#define __be32  uint32_t
#define __be16  uint16_t

#define BITS_PER_LONG __LONG_BIT

//#ifndef __be32_to_cpu
//#define __be32_to_cpu(x) ntohl(x)
//#endif
//#ifndef __cpu_to_be32
//#define __cpu_to_be32(x) htonl(x)
//#endif

#if __BYTE_ORDER == __LITTLE_ENDIAN
#ifndef __cpu_to_le32
#define  __cpu_to_le32(x) (x)
#endif
#ifndef __le32_to_cpu
#define  __le32_to_cpu(x) (x)
#endif
#elif __BYTE_ORDER == __BIG_ENDIAN
#ifndef __cpu_to_le32
#define  __cpu_to_le32(x) bswap_32(x)
#endif
#ifndef __le32_to_cpu
#define  __le32_to_cpu(x) bswap_32(x)
#endif
#else
#ifndef __cpu_to_le32
#define  __cpu_to_le32(x) bswap_32(__cpu_to_be32(x))
#endif
#ifndef __le32_to_cpu
#define  __le32_to_cpu(x) __be32_to_cpu(bswap_32(x))
#endif
#endif


/*
 * Byteswapping
 */
#define cpu_to_be16(x) htobe16(x)
#define cpu_to_be32(x) htobe32(x)
#define cpu_to_be64(x) htobe64(x)
#define be16_to_cpu(x) be16toh(x)
#define be32_to_cpu(x) be32toh(x)
#define be64_to_cpu(x) be64toh(x)

#define	_PATH_DEVPCI	"/dev/pci"
#define SLV_ADDRS_NUM 128

typedef enum MError {
    ME_OK = 0,
    ME_ERROR,
    ME_BAD_PARAMS,
    ME_CR_ERROR,
    ME_NOT_IMPLEMENTED,
    ME_SEM_LOCKED,
    ME_MEM_ERROR,

    ME_MAD_SEND_FAILED,
    ME_UNKOWN_ACCESS_TYPE,
    ME_UNSUPPORTED_DEVICE,
    ME_REG_NOT_SUPPORTED,

    // errors regarding REG_ACCESS
    ME_REG_ACCESS_OK = 0,
    ME_REG_ACCESS_BAD_STATUS_ERR = 0x100,
    ME_REG_ACCESS_BAD_METHOD,
    ME_REG_ACCESS_NOT_SUPPORTED,
    ME_REG_ACCESS_DEV_BUSY,
    ME_REG_ACCESS_VER_NOT_SUPP,
    ME_REG_ACCESS_UNKNOWN_TLV,
    ME_REG_ACCESS_REG_NOT_SUPP,
    ME_REG_ACCESS_CLASS_NOT_SUPP,
    ME_REG_ACCESS_METHOD_NOT_SUPP,
    ME_REG_ACCESS_BAD_PARAM,
    ME_REG_ACCESS_RES_NOT_AVLBL,
    ME_REG_ACCESS_MSG_RECPT_ACK,
    ME_REG_ACCESS_UNKNOWN_ERR,
    ME_REG_ACCESS_SIZE_EXCCEEDS_LIMIT,

    // errors regarding ICMD
    ME_ICMD_STATUS_CR_FAIL = 0x200,       // cr-space access failure
    ME_ICMD_INVALID_OPCODE,
    ME_ICMD_INVALID_CMD,
    ME_ICMD_OPERATIONAL_ERROR,
    ME_ICMD_BAD_PARAM,
    ME_ICMD_BUSY,
    ME_ICMD_INIT_FAILED,
    ME_ICMD_NOT_SUPPORTED,
    ME_ICMD_STATUS_SEMAPHORE_TO,          // timed out while trying to take semaphore
    ME_ICMD_STATUS_EXECUTE_TO,            // timed out while waiting for command to execute
    ME_ICMD_STATUS_IFC_BUSY,
    ME_ICMD_STATUS_ICMD_NOT_READY,
    ME_ICMD_UNSUPPORTED_ICMD_VERSION,
    ME_ICMD_UNKNOWN_STATUS,
    ME_ICMD_ICM_NOT_AVAIL,
    ME_ICMD_WRITE_PROTECT,

    //errors regarding Tools CMDIF
    ME_CMDIF_BUSY = 0x300,
    ME_CMDIF_TOUT,
    ME_CMDIF_BAD_STATUS,
    ME_CMDIF_BAD_OP,
    ME_CMDIF_NOT_SUPP,
    ME_CMDIF_BAD_SYS,
    ME_CMDIF_UNKN_TLV,

    //errors regarding MAD IF
    ME_MAD_BUSY = 0x400,
    ME_MAD_REDIRECT,
    ME_MAD_BAD_VER,
    ME_MAD_METHOD_NOT_SUPP,
    ME_MAD_METHOD_ATTR_COMB_NOT_SUPP,
    ME_MAD_BAD_DATA,
    ME_MAD_GENERAL_ERR,

    ME_LAST,
} MError;

typedef enum Mdevs_t {
    MDEVS_GAMLA     = 0x01, /*  Each device that actually is a Gamla */
    MDEVS_I2CM      = 0x02, /*  Each device that can work as I2C master */
    MDEVS_MEM       = 0x04, /*  Each device that is a memory driver (vtop) */
    MDEVS_TAVOR_DDR = 0x08, /*  Each device that maps to DDR */
    MDEVS_TAVOR_UAR = 0x10, /*  Each device that maps to UAR */
    MDEVS_TAVOR_CR  = 0x20, /*  Each device that maps to CR */
    MDEVS_IF        = 0x40, /*  Standard device  interface */
    MDEVS_REM       = 0x80, /*  Remote devices */
    MDEVS_PPC       = 0x100, /*  PPC devices */
    MDEVS_DEV_I2C   = 0x200, /* Generic linux kernel i2c device */
    MDEVS_IB        = 0x400, /* Cr access over IB Mads */
    MDEVS_MLNX_OS   = 0x800, /* access by CmdIf in MlnxOS */
    MDEVS_FWCTX		= 0x900, /*access by func/context (like uefi)*/
    MDEVS_TAVOR     = (MDEVS_TAVOR_DDR|MDEVS_TAVOR_UAR|MDEVS_TAVOR_CR),
    MDEVS_ALL       = 0xffffffff
} Mdevs;

typedef enum {
    MACCESS_REG_METHOD_GET = 1,
    MACCESS_REG_METHOD_SET = 2
} maccess_reg_method_t;

typedef enum DType_t {MST_GAMLA, MST_TAVOR, MST_DIMM, MST_NOADDR} DType;

typedef enum MType_t {MST_PCI, MST_PCICONF, /*MST_CALBR,*/ MST_USB,
                        MST_IB, MST_IF, MST_PPC, MST_USB_DIMAX, MST_MLNXOS, MST_FWCTX
#ifdef ENABLE_MST_DEV_I2C
                        ,MST_DEV_I2C
#endif
} MType;

typedef struct dev_info_t
{
    Mdevs         type;
    char          dev_name[512];

    union {
        struct {
            u_int16_t domain;
            u_int8_t  bus;
            u_int8_t  dev;
            u_int8_t  func;

            u_int16_t dev_id;
            u_int16_t vend_id;
            u_int32_t class_id;
            u_int16_t subsys_id;
            u_int16_t subsys_vend_id;

            char      cr_dev[512];
            char      conf_dev[512];
            char**    net_devs;      // Null terminated array
            char**    ib_devs;       // Null terminated array
        } pci;

        struct {
            u_int32_t TBD;
        } usb;

        struct {
            u_int32_t TBD;
        } ib;

        struct {
            u_int32_t TBD;
        } remote;
    };
} dev_info;


typedef struct mfile_l {
    MType         tp;           /*  type of driver */
    DType         dtype;        /*  target device to access to */
    int           fd;
    unsigned char i2c_slave;
    struct pcisel sel;
    void          *ptr;
    int           connectx_flush; /* For ConnectX A0 */
    int           need_flush; /* For ConnectX A0 */
    enum Mdevs_t  flags;
    dev_info*     dinfo;
} mfile;

void	    list_caps(int fd, struct pci_conf *p);
int         mtcr_check_signature(mfile *mf);
void        mtcr_connectx_flush(void *ptr);
mfile       *mopen(const char *name);
mfile       *mopend(char const *name, DType dtype);
mfile       *mopen_fw_ctx(void* fw_cmd_context, void* fw_cmd_func);
int         mclose(mfile *mf);
int         mtcr_open_config(mfile *mf, const char *name);
int         mread4(mfile *mf, unsigned int offset, u_int32_t *value);
int         mwrite4(mfile *mf, unsigned int offset, u_int32_t value);
int         mread4_block (mfile *mf, unsigned int offset, u_int32_t* data, int byte_len);
int         mwrite4_block (mfile *mf, unsigned int offset, u_int32_t* data, int byte_len);
int         mread64(mfile *mf, unsigned int offset, void *data, int length);
int         mwrite64(mfile *mf, unsigned int offset, void *data, int length);
int         msw_reset(mfile *mf);

uint32_t    read_config(int fd, struct pcisel *sel, unsigned int reg, int width);
void        write_config(int fd, struct pcisel *sel, unsigned int reg, unsigned int data, int width);

// Stubs
int mi2c_detect (mfile *mf, u_int8_t slv_arr[SLV_ADDRS_NUM]);
int mread_i2cblock(mfile *mf, unsigned char i2c_slave, u_int8_t addr_width,
                            unsigned int offset, void *data, int length);
int mwrite_i2cblock(mfile *mf, unsigned char i2c_slave, u_int8_t addr_width,
                    unsigned int offset, void *data, int length);

int mread_buffer(mfile *mf, unsigned int offset, u_int8_t* data, int byte_len);
int mwrite_buffer(mfile *mf, unsigned int offset, u_int8_t* data, int byte_len);

int mget_mdevs_flags(mfile *mf, u_int32_t *devs_flags);
int mget_mdevs_type(mfile *mf, u_int32_t *mtype);

/*
 * Set a new value for i2c_slave
 * Return previous value
 */
unsigned char mset_i2c_slave(mfile *mf, unsigned char new_i2c_slave);
int mget_i2c_slave(mfile *mf, unsigned char *new_i2c_slave_p);

/*
 *  * Get list of MST (Mellanox Software Tools) devices.
 *   * Put all device names as null-terminated strings to buf.
 *    *
 *     * Return number of devices found or -1 if buf overflow
 *      */
int mdevices(char *buf, int len, int mask);

dev_info* mdevices_info(int mask, int* len);
void mdevices_info_destroy(dev_info* dev_info, int len);

int maccess_reg(mfile     *mf,
                         u_int16_t  reg_id,
                         maccess_reg_method_t reg_method,
                         void*      reg_data,
                         u_int32_t  reg_size,
                         u_int32_t	r_size_reg, // used when sending via icmd interface (how much data should be read back to the user)
                         u_int32_t	w_size_reg, // used when sending via icmd interface (how much data should be written to the scratchpad)
                         	 	 	 	 	 	// if you dont know what you are doing then r_size_reg = w_size_reg = your_register_size
                         int       *reg_status);

/**
 * Handles the send command procedure.
 * for completeness, but calling it is strongly advised against.
 * @param[in] dev   A pointer to a device context, previously
 *                  obtained by a call to <tt>gcif_open</tt>.
 * @return          One of the GCIF_STATUS_* values, or a raw
 *                  status value (as indicated in cr-space).
 * NOTE: when calling this function the caller needs to make
 *  	sure device supports icmd.
 **/
int icmd_send_command(mfile    *mf,
                      int     opcode,
                      void*   data,
                      int     data_size,
                      int     skip_write);

/**
 * Clear the Tools-HCR semaphore. Use this when an application
 * that uses this library is not terminated cleanly, leaving the
 * semaphore in a locked state.
 * @param[in] dev   A pointer to a device context, previously
 *                  obtained by a call to <tt>gcif_open</tt>.
 * @return          One of the GCIF_STATUS_* values, or a raw
 *                  status value (as indicated in cr-space).
 * NOTE: when calling this function the caller needs to make
 *  	sure device supports icmd.
 **/
int icmd_clear_semaphore(mfile *mf);

/*
 * returns the maximal allowed register size (in bytes)
 * according to the FW access method or -1 if no restriction applicable
 *
 */
int mget_max_reg_size(mfile *mf);

const char* m_err2str(MError status);

int mib_smp_get(mfile* mf, u_int8_t* data, u_int16_t attr_id, u_int32_t attr_mod);
int mib_smp_set(mfile* mf, u_int8_t* data, u_int16_t attr_id, u_int32_t attr_mod);

int tools_cmdif_query_dev_cap(mfile *mf, u_int32_t offset, u_int64_t* data);

#ifdef __cplusplus
}
#endif


#endif
