#!/usr/local/bin/python

#--
#                 - Mellanox Confidential and Proprietary -
# 
# Copyright (C) Jan 2013, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
# 
# Except as specifically permitted herein, no portion of the information,
# including but not limited to object code and source code, may be reproduced,
# modified, distributed, republished or otherwise exploited in any form or by
# any means for any purpose without the prior written permission of Mellanox
# Technologies Ltd. Use of software subject to the terms and conditions
# detailed in the file "LICENSE.txt".
#--

"""
$Id           : mst.py 2013-11-19
$Authors      : Adham Masarwah adham@mellanox.com
$Tool         : MST tool replacement for FreeBSD
"""

try :
    import os
    import sys
    import subprocess
    import commands
    from subprocess import PIPE, STDOUT
    import re
    import tools_version
except Exception,e:
    print "-E- could not import some modules :",e
    sys.exit(1)
####################################################
toolVersion              = "1.0.0"
MFT_VERSION_STR     = "@MFT_VERSION_STR@"
TOOLS_BUILD_TIME    = "@TOOLS_BUILD_TIME@"
TOOLS_GIT_SHA       = "@TOOLS_GIT_SHA@"
PORT = 23108
####################################################    

def PrintVersionString(tool, ver):
    print '%s %s, %s, built on %s. GIT HASH: %s' % (tool, ver, MFT_VERSION_STR, TOOLS_BUILD_TIME, TOOLS_GIT_SHA)

usage_msg= "\n\tMST (Mellanox Software Tools) service\n"\
"  \t=====================================\n\n"\
"  This script is used to get MST devices \n\n"\
"  The mst commands are:\n"\
"  ----------------------\n\n"\
"  mst status  - Print current status of Mellanox devices.\n"\
"  mst help    - Print this help information.\n"\
"  mst version - Print mst version information.\n\n"\
"  mst server start [port]\n"\
"      Start MST server to allow incoming connection.\n"\
"      Default port is "+str(PORT)+".\n\n"\
"  mst server stop\n"\
"      Stop MST server." 

min_usage = "\nUsage:\n"\
"    mst {status|server start|server stop|help|version}\n\n"\
"Type \"mst help\" for detailed help\n"

def usage():
    print usage_msg

def isLivefish(devname):
    cmd = 'flint -d %s q' % devname
    process  = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    process.wait()
    out = process.communicate()[0]
    if "No valid image found" in out:
        return True
    return False

def mstStatus():
    cmd = 'pciconf -lv | grep -1 Mellanox'
    process  = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    process.wait()
    out = process.communicate()[0]
    device_ptrn = '[_\w]+\@(?P<device>pci\w+\:\w+:\w+\:\w+)'
    print 'MST devices:\n------------'
    for dev in re.finditer(device_ptrn, out):
        device = dev.groupdict()['device']
        device_info_ptrn = '[_\w]+\@%s.*\s+.*\s+device\s+=\s+\'(?P<info>.*)\'' % device
        device_info_r = re.search(device_info_ptrn, out)
        if isLivefish(device):
            device_info = "[Flash Recovery mode]"
        elif device_info_r == None:
            device_info = "Mellanox Family"
        else:
            device_info = device_info_r.groupdict()['info']
        print '  %s\t-\t%s' % (device, device_info)
        
    print ''
    return 0

def mst_server_start():
    cmd = ["mtserver", "-p", str(PORT)]
    try:
        pipe = subprocess.Popen(cmd, stdout=sys.stdout, stderr=subprocess.STDOUT)
        return 0
    except Exception, e:
        print >> sys.stderr, "-E- Failed to start mst server process. Got exception: %s" % str(e)
        return 1
        
def mst_server_stop():
    cmd = "ps | grep mtserver | grep -v grep"
    rc, output = commands.getstatusoutput(cmd)
    if output == "":
        print >> sys.stderr, "-E- No mtserver found running."
        return rc
    cmd = "pkill -9 mtserver"
    try:
        pipe = subprocess.Popen(cmd, stdout=sys.stdout, stderr=subprocess.STDOUT, shell=True)
        rc = pipe.wait()
        (stout, sterr) = pipe.communicate()
        if rc:
            print >> sys.stderr, "-E- %s" % str(sterr)
            return rc
        return 0
    except Exception, e:
        print >> sys.stderr, "-E- Failed to stop mst server process. Got error message: %s." % str(e)
        return 1

def exit_error():
    print '-E- Could not resolve command: %s, run help to see usage' % ' '.join(sys.argv[1:])
    sys.exit(1)        
        
if __name__ == "__main__":
    if len(sys.argv) < 2:
        print min_usage
        sys.exit(1)
    if "version" in sys.argv:
        tools_version.PrintVersionString("mst", toolVersion)
        sys.exit(0)
    if "help" in sys.argv:
        usage()
        sys.exit(0)
    if sys.argv[1] == 'status':
        func = mstStatus
    elif sys.argv[1] == "server" and len(sys.argv) == 3:
        if sys.argv[2] == "start":
            func = mst_server_start
        elif sys.argv[2] == "stop":
            func = mst_server_stop
        else:
            exit_error()
    elif sys.argv[1] == "server" and len(sys.argv) == 4:
        if sys.argv[2] == "start":
            try:
                PORT = int(sys.argv[3])
                func = mst_server_start
            except:
                exit_error()
    else:
        exit_error()
    
    sys.exit(int(func()))


